==> moonlight-1.20-2.16.15-fabric-sources.jar: net/mehvahdjukaar/moonlight/api/item/additional_placements/BlockPlacerItem.java <==
package net.mehvahdjukaar.moonlight.api.item.additional_placements;

import net.mehvahdjukaar.moonlight.api.MoonlightRegistry;
import net.minecraft.class_1269;
import net.minecraft.class_1747;
import net.minecraft.class_1750;
import net.minecraft.class_1792;
import net.minecraft.class_1838;
import net.minecraft.class_2248;
import net.minecraft.class_2498;
import net.minecraft.class_2680;
import net.minecraft.class_3414;
import net.minecraft.class_4174;
import org.jetbrains.annotations.Nullable;

import java.util.Map;

//hacky registered item that handles placing placeable stuff
public final class BlockPlacerItem extends class_1747 {

    public static BlockPlacerItem get() {
        return MoonlightRegistry.BLOCK_PLACER.get();
    }

    private class_4174 mimicFood;
    private class_2248 mimicBlock;
    private class_2498 overrideSound;

    public BlockPlacerItem(class_2248 pBlock, class_1793 pProperties) {
        super(pBlock, pProperties);
    }

    @Override
    public void method_7713(Map<class_2248, class_1792> pBlockToItemMap, class_1792 pItem) {
        AdditionalItemPlacementsAPI.onRegistryCallback(pBlockToItemMap);
    }


    @Nullable
    public class_2680 mimicGetPlacementState(class_1750 pContext, class_2248 toPlace) {
        this.mimicBlock = toPlace;
        var r = method_7707(pContext);
        this.mimicBlock = null;
        return r;
    }

    public class_1269 mimicUseOn(class_1838 pContext, class_2248 toPlace, @Nullable class_4174 foodProperties) {
        this.mimicFood = foodProperties;
        this.mimicBlock = toPlace;
        var r = super.method_7884(pContext);
        this.mimicFood = null;
        this.mimicBlock = null;
        return r;
    }

    public class_1269 mimicPlace(class_1750 pContext, class_2248 toPlace, @Nullable class_2498 overrideSound) {
        this.overrideSound = overrideSound;
        this.mimicBlock = toPlace;
        var r = super.method_7712(pContext);
        this.overrideSound = null;
        this.mimicBlock = null;
        return r;
    }

    @Override
    public class_2248 method_7711() {
        if (this.mimicBlock != null) return mimicBlock;
        return super.method_7711();
    }

    @Nullable
    @Override
    public class_4174 method_19264() {
        return mimicFood;
    }

    @Override
    public boolean method_19263() {
        return mimicFood != null;
    }

    @Override
    protected class_3414 method_19260(class_2680 pState) {
        if (this.overrideSound != null) return this.overrideSound.method_10598();
        return super.method_19260(pState);
    }

    @Override
    public boolean method_7709(class_1750 pContext, class_2680 pState) {
        this.mimicBlock = pState.method_26204();
        boolean r = super.method_7709(pContext, pState);
        this.mimicBlock = null;
        return r;
    }

    @Override
    public String method_7876() {
        return "x";
    }
}

==> moonlight-1.21-2.24.2-fabric-sources.jar: net/mehvahdjukaar/moonlight/api/item/additional_placements/BlockPlacerItem.java <==
package net.mehvahdjukaar.moonlight.api.item.additional_placements;

import net.mehvahdjukaar.moonlight.api.MoonlightRegistry;
import net.mehvahdjukaar.moonlight.api.misc.ForgeOverride;
import net.minecraft.class_1269;
import net.minecraft.class_1309;
import net.minecraft.class_1747;
import net.minecraft.class_1750;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1838;
import net.minecraft.class_2248;
import net.minecraft.class_2498;
import net.minecraft.class_2680;
import net.minecraft.class_3414;
import net.minecraft.class_4174;
import org.jetbrains.annotations.Nullable;

import java.util.Map;

//hacky registered item that handles placing placeable stuff
public final class BlockPlacerItem extends class_1747 {

    public static BlockPlacerItem get() {
        return MoonlightRegistry.BLOCK_PLACER.get();
    }

    private class_4174 mimicFood;
    private class_2248 mimicBlock;
    private class_2498 overrideSound;

    public BlockPlacerItem(class_2248 pBlock, class_1793 pProperties) {
        super(pBlock, pProperties);
    }

    @Override
    public void method_7713(Map<class_2248, class_1792> pBlockToItemMap, class_1792 pItem) {
    }


    @Nullable
    public class_2680 mimicGetPlacementState(class_1750 pContext, class_2248 toPlace) {
        this.mimicBlock = toPlace;
        var r = method_7707(pContext);
        this.mimicBlock = null;
        return r;
    }

    public class_1269 mimicUseOn(class_1838 pContext, class_2248 toPlace, class_4174 foodProperties) {
        this.mimicFood = foodProperties;
        this.mimicBlock = toPlace;
        var r = super.method_7884(pContext);
        this.mimicFood = null;
        this.mimicBlock = null;
        return r;
    }

    public class_1269 mimicPlace(class_1750 pContext, class_2248 toPlace, @Nullable class_2498 overrideSound) {
        this.overrideSound = overrideSound;
        this.mimicBlock = toPlace;
        var r = super.method_7712(pContext);
        this.overrideSound = null;
        this.mimicBlock = null;
        return r;
    }

    @Override
    public class_2248 method_7711() {
        if (this.mimicBlock != null) return mimicBlock;
        return super.method_7711();
    }

    @Nullable
    @ForgeOverride
    public class_4174 getFoodProperties(class_1799 stack, @Nullable class_1309 entity) {
        return mimicFood;
    }

    @Override
    protected class_3414 method_19260(class_2680 pState) {
        if (this.overrideSound != null) return this.overrideSound.method_10598();
        return super.method_19260(pState);
    }

    @Override
    public boolean method_7709(class_1750 pContext, class_2680 pState) {
        this.mimicBlock = pState.method_26204();
        boolean r = super.method_7709(pContext, pState);
        this.mimicBlock = null;
        return r;
    }

    @Override
    public String method_7876() {
        return "x";
    }
}
